/* The MazezaM Engine 
 * Copyright (C) 2002-2008 Malcolm Tyrrell
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package application;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.border.EmptyBorder;

/**
 * The panel which offers the user the key settings for a single Tessellation type.
 */
public class KeySettingsPanel extends JPanel {
	private String tabName;
	KeyField[] fields;
	int[] codes;

	/**
	 * 
	 */
	public KeySettingsPanel(KeySettingsDialog ksd, String tn,
			String[] keyNames, int[] keyCodes) {
		int numberOfDirections = keyCodes.length;
		JLabel label;
		fields = new KeyField[numberOfDirections];
		codes = keyCodes;
		tabName = tn;
		setLayout(new GridLayout(1, 2));

		int half = numberOfDirections / 2;
		for (int j = 0; j < 2; j++) {
			JPanel panel1 = new JPanel();
			panel1.setBorder(new EmptyBorder(2,2,2,2));
			GridBagLayout gridbag = new GridBagLayout();
			GridBagConstraints gbc = new GridBagConstraints();
			gbc.anchor = GridBagConstraints.WEST;
			gbc.fill = GridBagConstraints.BOTH;
			gbc.weighty = 1.0;
			panel1.setLayout(gridbag);
			KeyFieldKeyListener kcl = new KeyFieldKeyListener();
			for (int i = half*j; i < half*(j+1); i++) {
				label = new JLabel(keyNames[i] + "  ");
				gbc.gridwidth = GridBagConstraints.RELATIVE;
				gbc.weightx = 1.0;
				gridbag.setConstraints(label, gbc);
				panel1.add(label);
				fields[i] = new KeyField(keyCodes[i]);
				gbc.gridwidth = GridBagConstraints.REMAINDER;
				gbc.weightx = 0.0;
				gridbag.setConstraints(fields[i], gbc);
				fields[i].addKeyListener(kcl);
				panel1.add(fields[i]);
			}
			add(panel1);
		}
	}

	String getTabName() {
		return tabName;
	}

	void restore() {
		for (int i = 0; i < codes.length; i++)
			fields[i].setKeyCode(codes[i]);
	}

	void apply() {
		for (int i = 0; i < codes.length; i++) {
			codes[i] = fields[i].code;
		}
	}
}
