/* The MazezaM Engine 
 * Copyright (C) 2002-2008 Malcolm Tyrrell
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package file;

import globals.Globals;

/**
 * Exception thrown when parsing a level file
 */
public class LevelFileParseException extends Exception {
	protected int line;
	protected String message;
	protected String filename;

	LevelFileParseException(String m) {
		super(m);
		line = -1;
		message = m;
		filename = "";
	}

	LevelFileParseException(String m, int l) {
		super(m + Globals.text.getString("error_lineNumber") + l);
		line = l;
		message = m;
		filename = "";
	}

	LevelFileParseException(LevelParseException e, int l) {
		super(e.getMessage() + Globals.text.getString("error_lineNumber") + l
				+ e.getLine());
		line = l + e.getLine();
		message = e.getMessage();
		filename = "";
	}

	LevelFileParseException(String m, String f) {
		super(m + Globals.text.getString("error_fileOrUrl") + f);
		line = -1;
		message = m;
		filename = f;
	}

	void setFileName(String f) {
		filename = f;
	}

	public String getText1() {
		if (line == -1)
			return Globals.text.getString("error_cantOpen") + filename;
		else if (filename == "")
			return Globals.text.getString("error_parseError")
					+ Globals.text.getString("error_lineNumber") + line;
		else
			return Globals.text.getString("error_parseError")
					+ Globals.text.getString("error_fileOrUrl") + filename
					+ Globals.text.getString("error_lineNumber") + line;
	}

	public String getText2() {
		return message;
	}
}
