/* The MazezaM Engine 
 * Copyright (C) 2002-2008 Malcolm Tyrrell
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package gui_common;

import globals.Constants;
import globals.SettingsManager;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Insets;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.util.Observable;
import java.util.Observer;

import javax.swing.JPanel;
import javax.swing.border.Border;
import javax.swing.border.EmptyBorder;

import core.GameSwitcher;
import core.GameSwitcherInterface;

/**
 * The innermost panel in which games are displayed.
 */
class GamePanel extends JPanel implements Observer {

	// A hack to detect the first time the panel is drawn.
	private boolean initial = true;

	GameSwitcherInterface game_switcher;

	GamePanel(GameSwitcher gs) {
		setFocusable(true);

		setOpaque(true);
		setForeground(Color.white);
		setBackground(Color.black);

		// This is the object that manages the main window
		game_switcher = gs;

		gs.addObserver(this);

		addMouseListener(new MouseListener(game_switcher, this));
		addKeyListener(new KeyListener(game_switcher));
		
		Border lb = new EmptyBorder(new Insets(Constants.graphics_insets,
				Constants.graphics_insets, Constants.graphics_insets,
				Constants.graphics_insets));
		setBorder(lb);
	}

	public void paint(Graphics g) {
		// paint the background
		super.paint(g);

		/*
		 * FIXME There must be a better way! This is hacky: In order to ensure
		 * that the panel gets focus before the game chooser, I detect the first
		 * time it is drawn and request the focus then. This is the easiest way
		 * I can think of so that focus is transfered for applets.
		 */
		if (initial) {
			requestFocusInWindow();
			initial = false;
		}

		Graphics2D g2 = (Graphics2D) g;

		if (SettingsManager.graphics_antialiasing)
			g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
					RenderingHints.VALUE_ANTIALIAS_ON);

		game_switcher.paintPanel(g2, getInsets(), getSize());
	}

	/**
	 * This panel acts as an observer to GameManager objects.
	 */
	public void update(Observable o, Object arg) {
		if (arg instanceof Rectangle) {
			repaint((Rectangle) arg);
		}
		// else if (arg instanceof Boolean) {
		// if (((Boolean) arg).booleanValue())
		// repaint();
		// } else
		// throw new Error("unrecognised notification " + o + " " + arg);
		else {
			repaint();
		}
	}
}
