/* The MazezaM Engine 
 * Copyright (C) 2002-2008 Malcolm Tyrrell
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package gui_common;

import globals.Constants;
import globals.Globals;

import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Observable;
import java.util.Observer;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JPanel;
import javax.swing.JToolBar;
import javax.swing.event.PopupMenuEvent;
import javax.swing.event.PopupMenuListener;

import core.GameSwitcher;
import core.GameSwitcherInterface;

/**
 * Class for managing the bar of buttons
 */
/*
 * It observes a GameManager so that it can gray-out its buttons or not
 */
class ToolbarPanel extends JToolBar implements Observer, ActionListener,
		PopupMenuListener {
	GameSwitcherInterface game_panel;

	JButton next_button;
	JButton prev_button;
	JButton retry_button;
	JButton undo_button;
	JComboBox game_chooser;

	// used to manage the game_chooser
	int current_selection;
	
	ToolbarPanel(GameSwitcher gs) {
		// I just don't like floatable toolbars!
		setFloatable(false);
		JPanel panel = new JPanel(new FlowLayout(FlowLayout.LEFT,0,0));
		
		gs.addObserver(this);
		game_panel = gs;

		game_chooser = new JComboBox();
		game_chooser.setLightWeightPopupEnabled(true);
		game_chooser.setEditable(false);
		game_chooser.addActionListener(this);
		game_chooser.addPopupMenuListener(this);
		
		game_chooser.setFocusable(true);
		
		current_selection = 0;

		game_chooser.addItem(Globals.text.getString("toolbar_noItems"));

		panel.add(game_chooser);
		
		JPanel button_panel = new JPanel();
		button_panel.setLayout(new BoxLayout(button_panel,BoxLayout.X_AXIS));
		
		// None of these accept focus.

		prev_button = new JButton(
		// Constants.text_toolbar_retryButton);
				Globals.text.getString("toolbar_prevButton"));
		prev_button.addActionListener(this);
		prev_button.setMnemonic(Constants.mnemonic_prev);
		prev_button.setEnabled(false);
		prev_button.setFocusable(false);
		button_panel.add(prev_button);

		retry_button = new JButton(
		// Constants.text_toolbar_retryButton);
				Globals.text.getString("toolbar_retryButton"));
		retry_button.addActionListener(this);
		retry_button.setMnemonic(Constants.mnemonic_retry);
		retry_button.setEnabled(false);
		retry_button.setFocusable(false);
		button_panel.add(retry_button);

		next_button = new JButton(
		// Constants.text_toolbar_retryButton);
				Globals.text.getString("toolbar_nextButton"));
		next_button.addActionListener(this);
		next_button.setMnemonic(Constants.mnemonic_next);
		next_button.setEnabled(false);
		next_button.setFocusable(false);
		button_panel.add(next_button);

		undo_button = new JButton(Globals.text.getString("toolbar_undoButton"));
		undo_button.addActionListener(this);
		undo_button.setMnemonic(Constants.mnemonic_undo);
		undo_button.setEnabled(false);
		undo_button.setFocusable(false);
		button_panel.add(undo_button);
		
		panel.add(button_panel);
		add(panel);
		
	}

	private void changeChooser(String[] availableGames) {
		game_chooser.removeAllItems();
		if (availableGames.length > 0) {
			for (int i = 0; i < availableGames.length; i++)
				game_chooser.addItem(availableGames[i]);
			current_selection = -1;
		} else {
			game_chooser.addItem(Globals.text.getString("toolbar_noItems"));
		}
	}

	public void update(Observable o, Object arg) {
		if (o instanceof GameSwitcher) {
			prev_button.setEnabled(game_panel.can_goPrevious());
			retry_button.setEnabled(game_panel.can_retry());
			next_button.setEnabled(game_panel.can_goNext());
			undo_button.setEnabled(game_panel.can_undo());
			if (arg instanceof String[]) {
				changeChooser((String[]) arg);
			}
		} else
			throw new Error("unrecognised notification " + o + " " + arg);
	}

	public void actionPerformed(ActionEvent e) {
		if (e.getActionCommand() == Globals.text
				.getString("toolbar_prevButton"))
			game_panel.action_goPrevious();
		else if (e.getActionCommand() == Globals.text
				.getString("toolbar_retryButton"))
			game_panel.action_retry();
		else if (e.getActionCommand() == Globals.text
				.getString("toolbar_nextButton"))
			game_panel.action_goNext();
		else if (e.getActionCommand() == Globals.text
				.getString("toolbar_undoButton"))
			game_panel.action_undo();
		else if (e.getSource() instanceof JComboBox) {
			chooserAction(game_chooser.getSelectedIndex());
		}
	}

	private void chooserAction(int i) {
		if (current_selection != i) {
			game_panel.action_changeGame(i);
			game_chooser.setSelectedIndex(i);
			current_selection = i;
		}
	}

	public void popupMenuCanceled(PopupMenuEvent e) {
	}

	public void popupMenuWillBecomeInvisible(PopupMenuEvent e) {
		// This works but transferFocus() doesn't. Not sure why!
		transferFocusBackward();
	}

	public void popupMenuWillBecomeVisible(PopupMenuEvent e) {
	}
}
