/* The MazezaM Engine 
 * Copyright (C) 2002-2008 Malcolm Tyrrell
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package level;

/* Unit.java -
 *
 * Copyright (C) 2002-2005 Malcolm Tyrrell
 * malcohol@eircom.net
 *
 * This code may be used and distributed under the terms of the GNU General
 * Public Licence.
 */

/**
 * A static class which stores the standard character representation of units.
 */
public class Unit {
	// Used for both
	public static final char empty = ' ';

	// Used for shapes
	public static final char block = '$';
	public static final char wall = '#';
	public static final char person = '@';
	public static final char gate = '%';

	// Used for floors
	public static final char blank = '`';
	public static final char goal = '.';
	public static final char noBlocks = '_';
	public static final char noPeople = '!';

	// Meaningful overlaps
	public static final char persongoal = '+';
	public static final char blockgoal = '*';

	static char overlap(char first, char second) {
		// FIXME There must be a nicer way to do this!
		if (charIsEmpty(first))
			return second;
		else if ((first == goal) && (second == person))
			return persongoal;
		else if ((first == goal) && (second == block))
			return blockgoal;
		else if ((first == noBlocks) && (second == person))
			return person;
		else if (charIsEmpty(second))
			return first;
		else if ((second == goal) && (first == person))
			return persongoal;
		else if ((second == goal) && (first == block))
			return blockgoal;
		else if ((second == noBlocks) && (first == person))
			return person;
		else
			// if the execution falls through to here, there must be a
			// consistency error in the shape.
			// throw new Error("Attempted to overlap '"+first+"' and
			// '"+second+"'");
			// FIXME
			return first;
	}

	static boolean charIsEmpty(char c) {
		return (c == empty) || (c == blank);
	}

	protected static boolean charIsUnit(char c) {
		return (c == empty) || (c == block) || (c == wall) || (c == person)
				|| (c == gate) || (c == noBlocks) || (c == goal);
	}

	static boolean charIsFloorUnit(char c) {
		return (c == empty) || (c == noBlocks) || (c == goal) || (c == blank);
	}

	static boolean charIsShapeUnit(char c) {
		return (c == empty) || (c == block) || (c == wall) || (c == person)
				|| (c == gate);
	}

	static boolean charIsWallUnit(char c) {
		return (c == empty) || (c == wall);
	}
}
