/* The MazezaM Engine 
 * Copyright (C) 2002-2008 Malcolm Tyrrell
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package tessellation;

/**
 * Iterates over all the directions (representing edges) at a given position; It
 * starts at the edge best suited for finding the outer outline of a shape.
 */
public class EdgeIterator {
	private Direction original;
	private Direction current;
	private Position pos;

	public EdgeIterator(Position p) {
		Direction d = TessellationManager.newDirection();
		// ensure the starting position is an actual edge
		if (p.hasEdge(d))
			original = d;
		else
			original = d.nextEdgeAt(p);
		pos = p;
	}

	public boolean hasNext() {
		if (current == null)
			return true;
		else
			return (!(current.nextEdgeAt(pos).equals(original)));
	}

	public Direction next() {
		if (current == null)
			current = original;
		else
			current = current.nextEdgeAt(pos);
		return current;
	}
}